(function (Drupal, drupalSettings, lazyImageStyle) {

  /**
   * Returns true if IntersectionObserver not found else false.
   * @see https://github.com/w3c/IntersectionObserver/blob/master/polyfill/intersection-observer.js#L19
   *
   * @returns {boolean}
   */
  lazyImageStyle.attachPolyfill = function () {
    if ('IntersectionObserver' in window &&
      'IntersectionObserverEntry' in window &&
      'intersectionRatio' in window.IntersectionObserverEntry.prototype) {

      // Minimal polyfill for Edge 15's lack of `isIntersecting`
      // See: https://github.com/w3c/IntersectionObserver/issues/211
      if (!('isIntersecting' in window.IntersectionObserverEntry.prototype)) {
        Object.defineProperty(window.IntersectionObserverEntry.prototype,
          'isIntersecting', {
            get: function () {
              return this.intersectionRatio < 1;
            }
          });
      }
      return false;
    }
    return true;
  };

  /**
   * Override loadObject method.
   *
   * @param entries
   * @param observer
   */
  lazyImageStyle.loadObjects = function (entries, observer) {
    entries.forEach(function (entry) {
      if (entry.isIntersecting) {
        entry.target.src = entry.target.dataset.src;
        // Remove blue class if found.
        if (entry.target.classList.contains('blue')) {
          entry.target.classList.remove('blue');
        }
        observer.unobserve(entry.target);
      }
    })
  };

  /**
   * Override attach behavior method.
   *
   * @param context
   * @param settings
   */
  lazyImageStyle.attach = function (context, settings) {
    settings = settings || drupalSettings;
    if (this.attachPolyfill()) {
      let src_polyfill = settings.lazy_image_style.polyfill || false;
      if (src_polyfill) {
        // Attch observer if script loaded.
        if (document.querySelector('script[src="' + src_polyfill + '"]')) {
          lazyImageStyle.attachObserver();
        }
        else {
          var script = document.createElement('script');
          script.type = 'text/javascript';
          script.src = src_polyfill;
          script.onload = () => {
            lazyImageStyle.attachObserver();
          };
          document.head.appendChild(script);
        }
      }
      else {
        console.error(Drupal.t('Not found path to polyfill file.'))
      }
    }
    else {
      this.attachObserver();
    }
  };

  /**
   * Attach/Init observer.
   */
  lazyImageStyle.attachObserver = function () {
    let imgs = this.getObjects();
    let offset = Number(this.getConfig('offset', 100));
    let options = {
      rootMargin: `${offset}px`
    };


    this.observer = new IntersectionObserver(this.loadObjects, options);
    imgs.forEach(image => {
      this.observer.observe(image);
    })
  }

}(Drupal, drupalSettings, Drupal.behaviors.lazyImageStyle));
